# Quickstart adon

## Prerequisites

- **adon** understood [No!](https://gitlab.com/elioway/adon/blob/master/doc/abstract.md)
- **elioSin** prerequisites met? [No!](https://gitlab.com/elioway/eliosin/blob/master/doc/installing.md)
- You completed **eve** Quickstart [No!](https://gitlab.com/eliosin/eve/blob/master/doc/quickstart.md)
- You completed **god** Quickstart [No!](https://gitlab.com/eliosin/god/blob/master/doc/quickstart.md)
- You created a new **elioSin** theme called `genesis`.
- You're currently running `gulp` in its root folder and can see the page.

## TLDR Checklist

An adon adds and removes a class to a tagName based on simple user event. The name of the class should be camel formatted "adon" + Name, e.g. `adonFullName`.

1. Create the file `js/adon/adonFullName.js`

2. Write your logic - basically like this, but cleverer:

```
// NB: Twinned with /stylesheets/adon/adonFullName.css
// NB: Tested by /test/adon/adonFullName.js
jQuery.fn.extend({
adonFullName: function() {
return $(this).toggleClass("adonFullName")
}
});
```

3. Create the file `stylesheets/adon/adonFullName.scss`

4. Make the tagName the parent tag of the scss, e.g.

```
adonFullName {
// get sassy in here
}
```

5. Import the adonName into #{\$container} on `stylesheets/theme.scss`

6. Edit `js/main.js`

```
$(document).ready(function() {
$("nav").adonFullName(true);
});
```

7. Ensure tagName is in the correct _prong_ in `stylesheets/settings.scss`

Do steps **1** to **5** in one go:

```

yo sin:adon
```

## Exercise 1 HideOnScrollDown An adon which hides tagNames when the page scrolls down

We're going to create an adon which can be used to hide things when the page is scrolling down. You can then add this adon on to any tagNames which you want to hide or show depending on the scrolling behaviour of the user.

Create and edit `js/adon/adonHideOnScrollDown.js`, adding this code:

```javascript
// NB: Twinned with /stylesheets/adon/adonHideWhenScrollDown.scss
// NB: Tested by /test/adon/adonHideWhenScrollDown.js
var position = $(window).scrollTop()
jQuery.fn.extend({
  adonHideWhenScrollDown: function() {
    return $(this).each(function() {
      $(window).scroll(function() {
        var scroll = $(window).scrollTop()
        if (scroll > position) {
          $(this).addClass('adonHideWhenScrollDown')
        } else {
          $(this).removeClass('adonHideWhenScrollDown')
        }
        position = scroll
      })
      return $(this)
    })
  }
})
```

While page scrolls, all the tags connected to this adon will be given a new class `adonHideWhenScrollDown`. When the page starts scrolling up, the class is removed.

### What we learnt

1. That the adon's javascript is adding and removing classes based on user interaction.

2. How to write an adon which responds to the window scrolling.

## Exercise 2 Styling adonName classes

Create and edit `stylesheets/adon/adonHideOnScrollDown.scss`, adding this code:

```scss
// DEBUG: Twinned with /js/adon/adonHideWhenScrollDown.js
.adonHideWhenScrollDown {
  transition: visibility 0s linear $transition, all 0s;
  transition-timing-function: ease;
  visibility: hidden;
}
```

As the page is scrolling, remember, all the tags connected to this adon will temporarily has the class name `adonHideWhenScrollDown`. This short snippet of scss sets the `visibility: hidden;` of the `.adonHideWhenScrollDown` style, with a transition effect timed by the `$transition` setting.

### What we learnt

1. A adon's functionality is handled using css.

2. There's nothing to stop you writing `$(this).hide()` in an adon, but the elioWay would be to have `visibility: hidden;` in the adon's activated class.

3. Apparently, there's a `$transition` setting!

## Exercise 3 Pluging in adons into tagNames.

To plug your adon into a tagName, simply add them as lines in your `js/main.js` file:

```javascript
// js/main.js
$(document).ready(function() {
  $('nav').adonHideOnScrollDown(true)
  $('blockquote').adonHideOnScrollDown(true)
})
```

This will attach your _adon_ to the tagNames `aside` and `blockquote`.

### What we learnt

1. How to plug an adon into a tagName.

## Exercise 4 FullScreenOnClick An adon which makes an tag full screen on click.

This time we will use **elioGod**'s **yeoman** generator, **generator-sin** by calling `yo sin:adon`. This will quickly scaffold all the files needed for your adon, including the beginnings of a test. Enter `FullScreenOnClick` as your adonName.

```shell
yo sin:adon
```

### What yo sin:adon makes

Respond to the questions, especially choosing an adonName.

This quickstart will assume you chose the name `FullScreenOnClick`.

What yeoman generator produces follows **adon** s _pattern_ which is as follows:

```shell
project
│   ...
│
└───js
│   │   main.js
│   │   ...
│   │
│   └───adon
│       │   adonFullScreenOnClick.js
│       │   ...
│
└───stylesheets
│   │
│   └───adon
│       │   adonFullScreenOnClick.scss
│       │   ...
│
└───test
    │
    └───adon
        │-- adonFullScreenOnClick.js
        │   ...
```

### What we learnt

- `yo sin:adon` creates the `js/adon/adonName.js` script for your adonName.

- `yo sin:adon` creates `stylesheets/adon/adonName.scss` for styling a css class called `.adonName`.

- `yo sin:adon` creates `test/adon/adonName.js` for testing your adon.

- `yo sin:adon` adds `@import "adon/adonName.scss";` to `stylesheets/theme.scss`.

## Exercise 5 adons are about style

Start off by deleting `js/adon/adonFullScreenOnClick.js`. We won't need it.

Then add `$('figure').click().toggleClass(adonName)` the `js/main.js` file like this:

```
$(document).ready(function() {
  ...
  $('figure').click().toggleClass("adonFullScreenOnClick")
})
```

Now edit `stylesheets/adon/adonFullScreenOnClick.scss`:

```
.adonFullScreenOnClick {
  left: 0;
  margin: $personal-space;
  position: sticky;
  top: $personal-space;
  width: 100%;
}
```

### What we learnt

1. The scss file is the most important feature of an adon. You might not even need the js.

2. How to write an adon without an adonName script file.

## Exercise 6 Testing your adon

Type `yarn test` at the command line. We expect 1 test to fail. This is the test yo sin:adon created in a previous execise.

You'll see something like this:

```shell
5 passing (61ms)
1 failing

1) jQuery.fn.extend.adonFullScreenOnClick
     removes class adonFullScreenOnClick from tag:
   TypeError: $(...).adofFullScreenOnClick is not a function
    at Context.done (test/adon/adonFullScreenOnClick.js:26:17)
    at Context.done (test/suites/domjquery_suite.js:20:7)
```

The error "TypeError: \$(...).adofFullScreenOnClick is not a function" happens because we deleted the jscript file.

This is how we fix that test. Open `test/adon/adonFullScreenOnClick.js`.

Whereever you see `$(testQy).adonFullScreenOnClick();` change it to the same statement for activating it which we added to the `js/main.js` in the previous exercise.

`$(testQy).click().toggleClass("adonFullScreenOnClick");`

Remove the following two lines:

```
$(testQy).adofFullScreenOnClick();
$(testQy)[0].should.not.have.class("adonFullScreenOnClick"); // no class
```

Now you should have this:

```
const suites = require("../suites/domjquery_suite");

var testML = "<h1>PRETEST</h1>";
var testQy = "h1";

suites.LittleDomWithTheJJSuite(
  "jQuery.fn.extend.adonFullScreenOnClick",
  "../../js/adon/adonFullScreenOnClick.js",
  testML,
  function() {
    it("has no class when not be activated", done => {
      $(testQy)[0].should.not.have.class("adonFullScreenOnClick");
      done();
    });

    it("adds class adonFullScreenOnClick to tag", done => {
      $(testQy)[0].should.not.have.class("adonFullScreenOnClick");
      $(testQy).click().toggleClass("adonFullScreenOnClick");
      $(testQy)[0].should.have.class("adonFullScreenOnClick"); // true class
      done();
    });

    it("removes class adonFullScreenOnClick from tag", done => {
      $(testQy).click().toggleClass("adonFullScreenOnClick");
      $(testQy)[0].should.have.class("adonFullScreenOnClick");
      done();
    });
  }
);
```

Now run the test again using `yarn test`.

``

- [Testing adons](tutorial-testing.md)

## Tidy up

- Press `CTRL` and `C` to stop the browser.

- Type `yarn format` to make your work `prettier`.

- Type `yarn test`. You haven't written any tests yet, but there are 3 tests already which should always pass.

## Next

The quickstarts are complete. Now you have seen how to write adons for your elioSin themes.

Go into the world and preach the gospel.

## More

- [god Quickstart](https://gitlab.com/eliosin/eve/blob/master/doc/quickstart.md)
- [eve Quickstart](https://gitlab.com/eliosin/eve/blob/master/doc/quickstart.md)
- [Testing adons](tutorial-testing.md)
- [generator-sin Quickstart](https://gitlab.com/eliosin/generator-sin/blob/master/doc/quickstart.md)
- [Settings File](quickstart.settings-file.md)
- [Mixins](quickstart.mixins.md)
- [god Index](index.md)
